#include "ips_audio_alsa_ALSAMixer.h"





static const snd_pcm_format_t formats[] = {

		SND_PCM_FORMAT_S16_LE,
		SND_PCM_FORMAT_S16_BE,
		SND_PCM_FORMAT_U16_LE,
		SND_PCM_FORMAT_U16_BE,
		SND_PCM_FORMAT_S24_LE,
		SND_PCM_FORMAT_S24_BE,
		SND_PCM_FORMAT_U24_LE,
		SND_PCM_FORMAT_U24_BE,
		SND_PCM_FORMAT_S32_LE,
		SND_PCM_FORMAT_S32_BE,
		SND_PCM_FORMAT_U32_LE,
		SND_PCM_FORMAT_U32_BE,
		SND_PCM_FORMAT_MU_LAW,
		SND_PCM_FORMAT_A_LAW,
		SND_PCM_FORMAT_S24_3LE,
		SND_PCM_FORMAT_S24_3BE,
		SND_PCM_FORMAT_U24_3LE,
		SND_PCM_FORMAT_U24_3BE,
		SND_PCM_FORMAT_S20_3LE,
		SND_PCM_FORMAT_S20_3BE,
		SND_PCM_FORMAT_U20_3LE,
		SND_PCM_FORMAT_U20_3BE,
		SND_PCM_FORMAT_S18_3LE,
		SND_PCM_FORMAT_S18_3BE,
		SND_PCM_FORMAT_U18_3LE,
		SND_PCM_FORMAT_U18_3BE,
		SND_PCM_FORMAT_S32_BE,
		SND_PCM_FORMAT_S32_LE
};

static const unsigned int rates[] = {
		5512,
		8000,
		11025,
		16000,
		22050,
		32000,
		44100,
		48000,
		64000,
		88200,
		96000,
		176400,
		192000,
};



JNIEXPORT void JNICALL Java_ips_audio_alsa_ALSAMixer_enumeratePCMDevices(JNIEnv *env, jobject obj,jstring cardId,jstring devName,jint dev) {

	if(dev > MAXDEVICES){
		return;
	}
	snd_ctl_t *handle;
	snd_ctl_card_info_t *info;
	snd_pcm_info_t *pcminfo;
	snd_ctl_card_info_alloca(&info);
	snd_pcm_info_alloca(&pcminfo);

	jboolean iscopy;
	jclass class=(*env)->FindClass(env,"ips/audio/alsa/ALSAMixer");
	if(class==NULL){
		fprintf(stderr,"Java Class not found!");
		return;
	}
	jmethodID mId=(*env)->GetMethodID(env,class,"targetDatalineAvailable","(ZIIIIIIZZ)V");
	if(mId==NULL){
		fprintf(stderr,"Method not found!");
		return;
	}

	const char *cardIdC = (*env)->GetStringUTFChars(env,cardId, &iscopy);
	//printf("PCM query %s\n",cardIdC);
	snd_ctl_open(&handle, cardIdC, 0);
	snd_ctl_card_info(handle, info);

	//printf("PCM Device got info\n");
	if(dev != -1){

		snd_ctl_pcm_info(handle,pcminfo);
		snd_pcm_info_set_device(pcminfo,dev);
		snd_pcm_t *pcm;
		//	// TODO !
		//	char devName[100];
		//	sprintf(devName,"plug%s,%d",aname,dev);

		const char *devNameC = (*env)->GetStringUTFChars(env,devName, &iscopy);
		//printf("PCM Device %s\n",devNameC);

		int ret=snd_pcm_open(&pcm,devNameC,SND_PCM_STREAM_CAPTURE,0);

		if(ret>=0){
			//printf("Capture\n");

			// query supported formats
			snd_pcm_hw_params_t *hw_params;
			snd_pcm_hw_params_alloca(&hw_params);
			ret = snd_pcm_hw_params_any(pcm, hw_params);

			if(ret>=0){
				unsigned int min_channels;
				unsigned int max_channels;
				ret=snd_pcm_hw_params_get_channels_min(hw_params,&min_channels);
				ret=snd_pcm_hw_params_get_channels_max(hw_params,&max_channels);
				//printf("Channels: %d to %d\n",min_channels,max_channels);
				unsigned int minRate;
				unsigned int maxRate;
				ret=snd_pcm_hw_params_get_rate_min(hw_params,&minRate,NULL);
				ret=snd_pcm_hw_params_get_rate_max(hw_params,&maxRate,NULL);
				//printf("Samplerate: %d to %d\n",minRate,maxRate);
				jint bytes;
				jint bits;
				jboolean s;
				jboolean bigEndian;

				int i;
				for(i=0;i<ARRAY_SIZE(formats);i++){
					snd_pcm_format_t format=formats[i];
					//printf("Test format %d\n",format);
					ret=snd_pcm_hw_params_test_format(pcm,hw_params,format);
					if(ret==0){

						ret=snd_pcm_format_linear(format);
						// only PCM coded formats for now
						if(ret==1){
							//linear
							bytes=snd_pcm_format_size(format,1);

							bits=snd_pcm_format_width(format);
							s=(snd_pcm_format_signed(format));
							bigEndian=(snd_pcm_format_big_endian(format));
							if(DEBUG>4) printf("Format: %d %d %d %d\n",bytes,bits,s,bigEndian);
							(*env)->CallVoidMethod(env,obj,mId,JNI_TRUE,min_channels,max_channels,minRate,maxRate,bytes,bits,s,bigEndian);
						}

					}


				}


			}
			snd_pcm_close(pcm);
		}

		ret=snd_pcm_open(&pcm,devNameC,SND_PCM_STREAM_PLAYBACK,0);
		if(ret==0){
			//printf("Playback\n");
			snd_pcm_close(pcm);
		}
	}
}



// TODO crashes
JNIEXPORT void JNICALL Java_ips_audio_alsa_ALSAMixer_enumeratePCMLines(JNIEnv *env, jobject obj,jstring devName) {


	//snd_ctl_t *handle;
	snd_ctl_card_info_t *info;
	snd_pcm_info_t *pcminfo;
	snd_ctl_card_info_alloca(&info);
	snd_pcm_info_alloca(&pcminfo);

	jboolean iscopy;
	jclass class=(*env)->FindClass(env,"ips/audio/alsa/ALSAMixer");
	if(class==NULL){
		fprintf(stderr,"Java Class not found!");
		return;
	}
	jmethodID mId=(*env)->GetMethodID(env,class,"targetDatalineAvailable","(ZIIIIIIZZ)V");
	if(mId==NULL){
		fprintf(stderr,"Method not found!");
		return;
	}

	snd_pcm_t *pcm;
	const char *devNameC = (*env)->GetStringUTFChars(env,devName, &iscopy);
	//printf("PCM Device %s\n",devNameC);

	//			snd_config_t *sc;
	//
	//			if(snd_config){
	//			int ret=snd_config_search(snd_config,devNameC,&sc);
	//			if(ret>=0){
	//			ret=snd_pcm_open_lconf(&pcm,devNameC,SND_PCM_STREAM_CAPTURE,0,sc);
	int ret=snd_pcm_open(&pcm,devNameC,SND_PCM_STREAM_CAPTURE,0);
	if(ret>=0){
		//printf("Capture\n");

		// query supported formats
		snd_pcm_hw_params_t *hw_params;
		snd_pcm_hw_params_alloca(&hw_params);
		ret = snd_pcm_hw_params_any(pcm, hw_params);

		if(ret>=0){
			unsigned int min_channels;
			unsigned int max_channels;
			ret=snd_pcm_hw_params_get_channels_min(hw_params,&min_channels);
			ret=snd_pcm_hw_params_get_channels_max(hw_params,&max_channels);
			//printf("Channels: %d to %d\n",min_channels,max_channels);
			unsigned int minRate;
			unsigned int maxRate;
			ret=snd_pcm_hw_params_get_rate_min(hw_params,&minRate,NULL);
			ret=snd_pcm_hw_params_get_rate_max(hw_params,&maxRate,NULL);
			//printf("Samplerate: %d to %d\n",minRate,maxRate);
			jint bytes;
			jint bits;
			jboolean s;
			jboolean bigEndian;

			int i;
			for(i=0;i<ARRAY_SIZE(formats);i++){
				snd_pcm_format_t format=formats[i];
				//printf("Test format %d\n",format);
				ret=snd_pcm_hw_params_test_format(pcm,hw_params,format);
				if(ret==0){

					ret=snd_pcm_format_linear(format);
					// only PCM coded formats for now
					if(ret==1){
						//linear
						bytes=snd_pcm_format_size(format,1);

						bits=snd_pcm_format_width(format);
						s=(snd_pcm_format_signed(format));
						bigEndian=(snd_pcm_format_big_endian(format));
						if(DEBUG>4) printf("Format: %d %d %d %d\n",bytes,bits,s,bigEndian);
						(*env)->CallVoidMethod(env,obj,mId,JNI_FALSE,min_channels,max_channels,minRate,maxRate,bytes,bits,s,bigEndian);
					}

				}


			}


		}
		snd_pcm_close(pcm);
		//}
		//}
	}
}


JNIEXPORT void JNICALL Java_ips_audio_alsa_ALSAMixer_release(JNIEnv *env, jobject obj){
}



